import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import {ContactUsPageCmsRepository} from "./contact-us-page-cms.repository";
import {UpdateContactUsPageCmsDto} from "./dto/update-contact-us-page-cms.dto";
import {ContactUsPageCmsEntity} from "./entities/contact-us-page-cms.entity";
import {DoSpacesService} from "../../SpacesModule/SpacesService/doSpacesService";
import {UploadedMulterFileI} from "../../SpacesModule/SpacesService";

@Injectable()
export class ContactUsPageCmsService {
    constructor(
        @InjectRepository(ContactUsPageCmsRepository)
        private readonly repository: ContactUsPageCmsRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
        dto: UpdateContactUsPageCmsDto,
        files: {
            image?: UploadedMulterFileI
        }
    ): Promise<ContactUsPageCmsEntity | Error> {
        if (files && files.image) {
            const image: any = await this.doSpaceService.uploadFile(files.image[0], "ContactUsPageCMS/");
            dto.image = image;
        }
        const isExists = await this.repository.find({isDeleted: false});
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<ContactUsPageCmsEntity[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}