import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import {HomePageCmsRepository} from "./home-page-cms.repository";
import {UpsertHomePageCmsDto} from "./dto/upsert-home-page-cms.dto";
import {HomePageCms} from "./entities/home-page-cms.entity";
import {FileUploadsService} from "src/modules/file-uploads/file-uploads.service";
import {DoSpacesService} from "../../SpacesModule/SpacesService/doSpacesService";

@Injectable()
export class HomePageCmsService {
    constructor(
        @InjectRepository(HomePageCmsRepository)
        private readonly repository: HomePageCmsRepository,
        private readonly fileService: FileUploadsService,
        private readonly doSpacesService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
        dto: UpsertHomePageCmsDto
    ): Promise<HomePageCms | Error> {
        const isExists = await this.repository.find();
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.GENERIC_ERROR,
                    HttpStatus.FORBIDDEN
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.UpdateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.FORBIDDEN
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
        }
    }

    async findAll(): Promise<HomePageCms[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}
