import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import {PrivacyPolicyCmsRepository} from "./privacy-policy-cms.repository";
import {UpdatePrivacyPolicyCmsDto} from "./dto/update-privacy-policy-cms.dto";
import {PrivacyPolicyCms} from "./entities/privacy-policy-cms.entity";
import {DoSpacesService} from "../../SpacesModule/SpacesService/doSpacesService";
import {UploadedMulterFileI} from "../../SpacesModule/SpacesService";

@Injectable()
export class PrivacyPolicyCmsService {
    constructor(
        @InjectRepository(PrivacyPolicyCmsRepository)
        private readonly repository: PrivacyPolicyCmsRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
        dto: UpdatePrivacyPolicyCmsDto,
        files: {
            logo: UploadedMulterFileI
        }
    ): Promise<PrivacyPolicyCms | Error> {
        if (files && files.logo) {
            const logo: any = await this.doSpaceService.uploadFile(files.logo[0], "PrivacyPolicyCMS/");
            dto.logo = logo;
        }
        const isExists = await this.repository.find({isDeleted: false});
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<PrivacyPolicyCms[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}