import {EntityRepository, getConnection, Repository} from "typeorm";
import {UpdateStoreCmsDto} from "./dto/update-store-cms.dto";
import {StoreCms} from "./entities/store-cms.entity";

@EntityRepository(StoreCms)
export class StoreCmsRepository extends Repository<StoreCms> {

    async createEntity(dto: UpdateStoreCmsDto): Promise<StoreCms | Error> {
        try {
            const entity = this.create({
                name: dto.name,
                address: dto.address,
                retailerId: dto.retailerId
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async updateEntity(dto: UpdateStoreCmsDto, id: string): Promise<StoreCms | Error> {
        try {
            await this.update(id, {
                name: dto.name,
                address: dto.address,
                retailerId: dto.retailerId
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findAll(): Promise<StoreCms[] | Error> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(StoreCms)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}