import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import {Constants} from "src/utils/constants";
import {AboutUsSectionTwoService} from "./about-us-section-two.service";
import {CreateAboutUsSectionTwoDto} from "./dto/create-about-us-section-two.dto";
import {UpdateAboutUsSectionTwoDto} from "./dto/update-about-us-section-two.dto";
import {AdminAuthGuard} from "src/core/guards/admin-auth.guard";
import {FileFieldsInterceptor} from "@nestjs/platform-express";
import {UploadedMulterFileI} from "../SpacesModule/SpacesService";

@Controller({path: "about-us-section-two", version: Constants.API_VERSION_1})

export class AboutUsSectionTwoController {
    constructor(private readonly service: AboutUsSectionTwoService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: "icon", maxCount: 1}
        ]
    ))
    create(@Body() dto: CreateAboutUsSectionTwoDto,
           @UploadedFiles() files: {
               icon: UploadedMulterFileI,
           }) {
        return this.service.create(dto, files);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch("/:id")
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: "icon", maxCount: 1}
        ]
    ))
    update(
        @Param("id") id: string,
        @Body() dto: UpdateAboutUsSectionTwoDto,
        @UploadedFiles() files: {
            icon: UploadedMulterFileI,
        }) {
        return this.service.update(id, dto, files);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
