import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateAboutUsSectionTwoDto} from "./dto/create-about-us-section-two.dto";
import {UpdateAboutUsSectionTwoDto} from "./dto/update-about-us-section-two.dto";
import {AboutUsSectionTwo} from "./entities/about-us-section-two.entity";

@EntityRepository(AboutUsSectionTwo)
export class AboutUsSectionTwoRepository extends Repository<AboutUsSectionTwo> {
    async createEntity(dto: CreateAboutUsSectionTwoDto): Promise<AboutUsSectionTwo | Error> {
        try {
            const {
                title,
                description,
                icon
            } = dto;
            const entity = this.create({
                title,
                description,
                icon
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<AboutUsSectionTwo | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateAboutUsSectionTwoDto): Promise<AboutUsSectionTwo | Error> {
        try {
            await this.update(id, {
                title: dto.title,
                description: dto.description,
                icon: dto.icon
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<AboutUsSectionTwo[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(AboutUsSectionTwo)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}