import {Body, Controller, Delete, Get, Param, Patch, UploadedFiles, UseInterceptors} from "@nestjs/common";
import {Constants} from "src/utils/constants";
import {RetailerServiceService} from "./retailer-service.service";
import {FileFieldsInterceptor} from "@nestjs/platform-express";
import {UploadedMulterFileI} from "../../../SpacesModule/SpacesService";

@Controller({path: "retailer-service", version: Constants.API_VERSION_1})

export class RetailerServiceController {
    constructor(private readonly service: RetailerServiceService) {
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    // @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }


    @Patch('/alpineiq-update/:id')
    updateAlpineIq(@Param('id') id: string, @Body() dto: any) {
        return this.service.updateAlpineIq(id, dto);
    }


    @Patch(":id")
    // @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: "retailerImage", maxCount: 1}
        ]
    ))
    update(
        @Param("id") id: string,
        @Body() dto: any,
        @UploadedFiles() files: {
            retailerImage?: UploadedMulterFileI
        }
    ) {
        return this.service.update(id, dto, files);
    }

    @Delete(":id")
    // @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
