import {Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {ResponseUtils} from "src/utils/response.utils";
import {CareerRepository} from "../career/career.repository";
import {CareerDepartmentRepository} from "../career-department/career-department.repository";
import {Between, FindOperator, Like} from "typeorm";
import {CareerListPageCmsRepository} from "../CMS/career-list-page-cms/career-list-page-cms.repository";
import {CareerDetailsPageCmsRepository} from "../CMS/career-details-page-cms/career-details-page-cms.repository";
import {WhyJoinUsRepository} from "../why-join-us/why-join-us.repository";
import {
    ScheduleVendorDayBannerCmsRepository
} from "../CMS/schedule-vendor-day-banner-cms/schedule-vendor-day-banner-cms.repository";
import {BlogCategoryRepository} from "../blog/blog-category/blog-category.repository";
import {BlogTagRepository} from "../blog/blog-tag/blog-tag.repository";
import {BlogPostRepository} from "../blog/blog-post/blog-post.repository";
import {UpdateBlogPostDto} from "../blog/blog-post/dto/update-blog-post.dto";
import {BlogPostTagsRepository} from "../blog/blog-post-tags/blog-post-tags.repository";
import {FAQRepository} from "../faq/faq.repository";
import {FaqPageSectionTwoRepository} from "../faq-page-section-two/faq-page-section-two.repository";
import {FAQPageCmsRepository} from "../CMS/faq-page-cms/faq-page-cms.repository";
import {ContactUsPageCmsRepository} from "../CMS/contact-us-page-cms/contact-us-page-cms.repository";
import {ContactUsSectionOneRepository} from "../contact-us-section-one/contact-us-section-one.repository";
import {AboutUsPageCmsRepository} from "../CMS/about-us-page-cms/about-us-page-cms.repository";
import {AboutUsSectionOneCmsRepository} from "../CMS/about-us-section-one-cms/about-us-section-one-cms.repository";
import {AboutUsSectionTwoRepository} from "../about-us-section-two/about-us-section-two.repository";
import {DispoDifferenceSectionRepository} from "../dispo-difference-section/dispo-difference-section.repository";
import {NewsletterBannerCmsRepository} from "../CMS/newsletter-banner-cms/newsletter-banner-cms.repository";
import {ReeferFriendCmsRepository} from "../CMS/reefer-friend-cms/reefer-friend-cms.repository";
import {ReeferFriendRepository} from "../reefer-friend/reefer-friend.repository";
import {HeroSectionLeftCmsRepository} from "../CMS/hero-section-left-cms/hero-section-left-cms.repository";
import {HeroSectionSliderRepository} from "../hero-section-slider/hero-section-slider.repository";
import {HomePageCmsRepository} from "../CMS/home-page-cms/home-page-cms.repository";
import {CareerApplicantRepository} from "../career-applicant/career-applicant.repository";
import {ContactMessageRepository} from "../contact-message/contact-message.repository";
import {RetailerService} from "../dutchie-plus/retailer/retailer.service";
import {RetailerServiceRepository} from "../dutchie-plus/custom/retailer-service/retailer-service.repository";
import {NewsletterRepository} from "../newsletter/newsletter.repository";

@Injectable()
export class SiteFrontendService {
    constructor(
        @InjectRepository(CareerRepository)
        private readonly careerRepository: CareerRepository,
        @InjectRepository(CareerDepartmentRepository)
        private readonly careerDepartmentRepository: CareerDepartmentRepository,
        @InjectRepository(CareerListPageCmsRepository)
        private readonly careerListPageCmsRepository: CareerListPageCmsRepository,
        @InjectRepository(CareerDetailsPageCmsRepository)
        private readonly careerDetailsPageCmsRepository: CareerDetailsPageCmsRepository,
        @InjectRepository(WhyJoinUsRepository)
        private readonly whyJoinUsRepository: WhyJoinUsRepository,
        @InjectRepository(ScheduleVendorDayBannerCmsRepository)
        private readonly scheduleVendorDayBannerCmsRepository: ScheduleVendorDayBannerCmsRepository,
        @InjectRepository(BlogCategoryRepository)
        private readonly blogCategoryRepository: BlogCategoryRepository,
        @InjectRepository(BlogTagRepository)
        private readonly blogTagRepository: BlogTagRepository,
        @InjectRepository(BlogPostRepository)
        private readonly blogPostRepository: BlogPostRepository,
        @InjectRepository(BlogPostTagsRepository)
        private readonly blogPostTagsRepository: BlogPostTagsRepository,
        @InjectRepository(FAQRepository)
        private readonly faqRepository: FAQRepository,
        @InjectRepository(FaqPageSectionTwoRepository)
        private readonly faqSecTwoRepository: FaqPageSectionTwoRepository,
        @InjectRepository(FAQPageCmsRepository)
        private readonly faqCmsRepository: FAQPageCmsRepository,
        @InjectRepository(ContactUsPageCmsRepository)
        private readonly contactUsPageCmsRepository: ContactUsPageCmsRepository,
        @InjectRepository(ContactUsSectionOneRepository)
        private readonly contactUsSectionOneRepository: ContactUsSectionOneRepository,
        @InjectRepository(AboutUsPageCmsRepository)
        private readonly aboutUsPageCmsRepo: AboutUsPageCmsRepository,
        @InjectRepository(AboutUsSectionOneCmsRepository)
        private readonly aboutUsSectionOneCmsRepo: AboutUsSectionOneCmsRepository,
        @InjectRepository(AboutUsSectionTwoRepository)
        private readonly aboutUsSectionTwoRepo: AboutUsSectionTwoRepository,
        @InjectRepository(DispoDifferenceSectionRepository)
        private readonly dispoDifferenceSectionRepo: DispoDifferenceSectionRepository,
        @InjectRepository(NewsletterBannerCmsRepository)
        private readonly newsletterBannerCmsRepository: NewsletterBannerCmsRepository,
        @InjectRepository(ReeferFriendCmsRepository)
        private readonly reeferFriendCmsRepository: ReeferFriendCmsRepository,
        @InjectRepository(ReeferFriendRepository)
        private readonly reeferFriendRepository: ReeferFriendRepository,
        @InjectRepository(HeroSectionLeftCmsRepository)
        private readonly heroSectionLeftCmsRepository: HeroSectionLeftCmsRepository,
        @InjectRepository(HeroSectionSliderRepository)
        private readonly heroSectionSliderRepository: HeroSectionSliderRepository,
        @InjectRepository(HomePageCmsRepository)
        private readonly homePageCmsRepository: HomePageCmsRepository,
        @InjectRepository(CareerApplicantRepository)
        private readonly careerApplicantRepository: CareerApplicantRepository,
        @InjectRepository(ContactMessageRepository)
        private readonly contactMessageRepository: ContactMessageRepository,
        private readonly retailerService: RetailerService,
        @InjectRepository(RetailerServiceRepository)
        private readonly retailerServiceRepository: RetailerServiceRepository,
        @InjectRepository(NewsletterRepository)
        private readonly newsletterRepository: NewsletterRepository
    ) {
    }

    async getCareerPageData(): Promise<any | Error> {
        const careerDepartmentList = await this.careerDepartmentRepository.findByFilterQuery({});
        const totalCount = await this.careerRepository.totalCareerCount();
        let careerDepartmentData: any = [];
        for (let i = 0; i < Object.keys(careerDepartmentList).length; i++) {
            const careerCount = await this.careerRepository.countByDepartmentName(careerDepartmentList[i].name);
            careerDepartmentList[i]["count"] = careerCount;
            careerDepartmentData.push(careerDepartmentList[i]);
        }

        const response = {
            careerDepartmentData: careerDepartmentData,
            totalCount: totalCount
        };

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully!", "data", response);
    }

    async getCareerDetailsData(id: string): Promise<any | Error> {
        const cmsData = await this.careerDetailsPageCmsRepository.findAll();
        const careerDetailsData = await this.careerRepository.findByFilterQuery({id: id});
        const whyJoinUsData = await this.whyJoinUsRepository.findByFilterQuery({});
        const latestCareerList = await this.careerRepository.getLatestCareerList();

        const response = {
            cmsData: cmsData,
            careerDetailsData: careerDetailsData,
            whyJoinUsData: whyJoinUsData,
            latestCareerList: latestCareerList
        };

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", response);
    }

    async getCareerDepartmentWiseData(
        departmentNameSlug: any,
        page: number,
        limit: number
    ): Promise<any | Error> {
        type FindOption = {
            departmentName?: FindOperator<string>;
        };

        const findOptions: FindOption = {};

        let departmentData = await this.careerDepartmentRepository.findBySlug(departmentNameSlug);
        let departmentName: any;
        if (Object.keys(departmentData).length > 0) {
            departmentName = departmentData[0].name;
        }

        if (departmentName) findOptions.departmentName = departmentName;

        const cmsData = await this.careerListPageCmsRepository.findAll();
        const whyJoinUsData = await this.whyJoinUsRepository.findByFilterQuery({});
        const careerList = await this.careerRepository.findAllWithPagination(findOptions, page, limit);
        const isCareerDepartmentExist = await this.careerDepartmentRepository.findOne({
            where: {
                name: departmentName,
                isDeleted: false
            }
        });

        let data = {
            cmsData: cmsData,
            careerList: careerList,
            whyJoinUsData: whyJoinUsData,
            isCareerDepartmentExist: typeof isCareerDepartmentExist !== "undefined" ? true : false
        };

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async getBlogData(
        page: number,
        limit: number,
        keyword: any,
        tagName: any,
        categoryId: any,
        date?: Date
    ): Promise<any | Error> {
        type FindOption = {
            title?: FindOperator<string>;
            categoryId?: FindOperator<string>;
            createdAt?: FindOperator<Date>;
        };

        let creationDate: any;

        const findOptions: FindOption = {};

        if (keyword) findOptions.title = Like("%" + keyword + "%");
        if (categoryId) findOptions.categoryId = categoryId;
        if (date) creationDate = date;

        const startDate = new Date(creationDate);
        startDate.setUTCHours(0, 0, 0, 0);
        const endDate = new Date(creationDate);
        endDate.setUTCHours(23, 59, 59, 999);

        startDate.toDateString();
        endDate.toDateString();

        if (date) {
            findOptions.createdAt = Between(startDate, endDate);
        }

        if (tagName) {
            const scheduleVendorDayData = await this.scheduleVendorDayBannerCmsRepository.findAll();
            const blogCategoryList = await this.blogCategoryRepository.findByFilterQuery({});
            const blogTagList = await this.blogTagRepository.findByFilterQuery({});
            const topBlogList = await this.blogPostRepository.getTopBlogList();
            const totalBlogCount = await this.blogPostRepository.countAllBlogs();
            const blogList = await this.blogPostRepository.findBlogByTagName(
                page,
                limit,
                tagName
            );

            let data = {
                scheduleVendorDayData: scheduleVendorDayData,
                blogCategoryList: blogCategoryList,
                blogTagList: blogTagList,
                topBlogList: topBlogList,
                blogList: blogList,
                totalBlogCount: totalBlogCount
            };

            return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
        } else {
            const scheduleVendorDayData = await this.scheduleVendorDayBannerCmsRepository.findAll();
            const blogCategoryList = await this.blogCategoryRepository.findByFilterQuery({});
            const blogTagList = await this.blogTagRepository.findByFilterQuery({});
            const topBlogList = await this.blogPostRepository.getTopBlogList();
            const totalBlogCount = await this.blogPostRepository.countAllBlogs();
            const blogList = await this.blogPostRepository.findBlogByFilterQuery(
                page,
                limit,
                findOptions
            );

            let data = {
                scheduleVendorDayData: scheduleVendorDayData,
                blogCategoryList: blogCategoryList,
                blogTagList: blogTagList,
                topBlogList: topBlogList,
                blogList: blogList,
                totalBlogCount: totalBlogCount
            };

            return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
        }
    }

    async getBlogDetailsData(urlSlug: string): Promise<any | Error> {
        const scheduleVendorDayData = await this.scheduleVendorDayBannerCmsRepository.findAll();
        const blogData = await this.blogPostRepository.findOneEntityBySlug(urlSlug);
        let id: any;
        if (Object.keys(blogData).length > 0) {
            id = blogData[0].id;
            let viewCount = 0;
            let totalViews = blogData[0].totalViews;
            let postId = blogData[0].id;
            viewCount = totalViews + 1;
            let blogPostDto = {
                totalViews: viewCount
            };
            await this.blogPostRepository.updateBlogViewCount(
                postId,
                blogPostDto as UpdateBlogPostDto
            );
        }
        const blogTags = await this.blogPostTagsRepository.findBlogPostTagsByPostId(id);
        if (blogTags) {
            blogData[0]["tags"] = blogTags;
        }

        const blogCategoryList = await this.blogCategoryRepository.findByFilterQuery({});
        const blogTagList = await this.blogTagRepository.findByFilterQuery({});
        const topBlogList = await this.blogPostRepository.getTopBlogList();

        let data = {
            scheduleVendorDayData: scheduleVendorDayData,
            blogData: blogData,
            blogCategoryList: blogCategoryList,
            blogTagList: blogTagList,
            topBlogList: topBlogList
        };

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async getFaqPageData(): Promise<any | Error> {
        const faqCms = await this.faqCmsRepository.findAll();
        const faqSectionTwo = await this.faqSecTwoRepository.findByFilterQuery({});
        const faq = await this.faqRepository.findByFilterQuery({});

        const response = {faqCms, faqSectionTwo, faq};

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully!", "data", response);
    }

    async getContactUsPageData(): Promise<any | Error> {
        const cmsData = await this.contactUsPageCmsRepository.findAll();
        const sectionOneData = await this.contactUsSectionOneRepository.findByFilterQuery({});

        const response = {
            cmsData: cmsData,
            sectionOneData: sectionOneData
        };

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully!", "data", response);
    }

    async getAboutUsPageData(): Promise<any | Error> {
        const aboutUsPageCms = await this.aboutUsPageCmsRepo.findAll();
        const aboutUsSectionOneCms = await this.aboutUsSectionOneCmsRepo.findAll();
        const aboutUsSectionTwo = await this.aboutUsSectionTwoRepo.findByFilterQuery({});
        const dispoDifferenceSection = await this.dispoDifferenceSectionRepo.findByFilterQuery({});
        const newsletterBannerCms = await this.newsletterBannerCmsRepository.findAll();

        const response = {
            aboutUsPageCms,
            aboutUsSectionOneCms,
            aboutUsSectionTwo,
            dispoDifferenceSection,
            newsletterBannerCms
        };

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully!", "data", response);
    }


    async home(): Promise<any | Error> {
        const reeferFriendCms = await this.reeferFriendCmsRepository.findAll();
        const reeferFriend = await this.reeferFriendRepository.findByFilterQuery({});
        const heroSectionLeftCms = await this.heroSectionLeftCmsRepository.findAll();
        const heroSectionSlider = await this.heroSectionSliderRepository.findByFilterQuery({});
        const homePageCms = await this.homePageCmsRepository.findAll();
        const dispoDifference = await this.dispoDifferenceSectionRepo.findByFilterQuery({});
        const newsletterCMS = await this.newsletterBannerCmsRepository.findAll();
        const response = {
            reeferFriendCms,
            reeferFriend,
            heroSectionLeftCms,
            heroSectionSlider,
            homePageCms,
            dispoDifference,
            newsletterCMS
        };

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully!", "data", response);
    }

    async getDashboardData(): Promise<any | Error> {
        const totalCareerApplicant = await this.careerApplicantRepository.countAllQuery();
        const totalContact = await this.contactMessageRepository.countAllQuery();
        const totalSubscriber = await this.newsletterRepository.countAllQuery();
        const totalArticle = await this.blogPostRepository.countAllBlogs();
        const totalCareer = await this.careerRepository.totalCareerCount();
        const totalRetailer = await this.retailerService.countAllRetailer();
        const topFiveRetailers = await this.retailerServiceRepository.topFiveRetailerEntity();
        let retailerLabel = [];
        let retailerAccessCount = [];
        for (let i = 0; i < Object.keys(topFiveRetailers).length; i++) {
            retailerLabel.push(topFiveRetailers[i].retailerName);
            retailerAccessCount.push(topFiveRetailers[i].accessCount);
        }
        const topFiveRetailerData = {
            labels: retailerLabel,
            data: retailerAccessCount
        }

        const response = {
            totalCareerApplicant: totalCareerApplicant,
            totalCareer: totalCareer,
            totalContact: totalContact,
            totalArticle: totalArticle,
            totalRetailer: totalRetailer,
            topFiveRetailer: topFiveRetailerData,
            totalSubscriber: totalSubscriber
        };

        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", response);
    }
}
